// This script, from https://ss.deviatenow.com, is protected by this licence :
// CC by-NC, see http://creativecommons.org/licenses/by-nc/3.0/

setInfos(9, "Shock Yourself Macro Maker", "Audacity Macro Maker",
        "MissEshock2002",
        "v0.3", 0x222222, "en", ["estim", "electro", "formale", "forfemale", "pain"]);

// TODO many frequencys
// TODO burst frequency, burst length
// TODO length input


show(null)
setImage(null)

List<String> WAVEFORMS = ["Sawtooth", "Sine", "Square"]
String PATHTOAUDACITYMACROS = "/home/?/.audacity-data/Macros/"

def dir = getDataFolder()
def pathToAudacityMacros = PATHTOAUDACITYMACROS
String macroName = "ShockYourself.txt"
String outputPath = dir + "sounds/ShockYourself/"
def waveforms = WAVEFORMS.collect()
String extention = "wav"
int length = 10
int frequency = 100
//def frequencys = [100, 200, 300, 400, 500]
int minPainLevel = 1
int maxPainLevel = 20
def minAmplitude = 0.0
def stepAmplitude = 1.0 / maxPainLevel // = 0.05
float noise = 1 / 2 * stepAmplitude // = 0.025

String macroText = ""
String fileName = ""
float amplitude
int track
boolean makeBurst = true

def helpTextMacro =
        "\nHow to use a macro:" +
                "\n- open Audacity" +
                "\n- Tools > Macros..." +
                "\n- select the macro" +
                "\n- click 'Project'" +
                "\n- wait (can take long)" +
                "\n- Audacity can be closed without saving"


// Macro templates
String waveTemplate = "NewMonoTrack:\n" +
        "Select:End=\"%s\" Mode=\"Set\" Start=\"0\" Track=\"%s\"\n" +
        "Tone:Amplitude=%s Frequency=%s Interpolation=\"Linear\" Waveform=%s\n"

String burstTemplate = "NewMonoTrack:\n" +
        "Tone:Amplitude=\"%s\" Frequency=\"%s\" Interpolation=\"Linear\" Waveform=\"%s\"\n" +
        "Select:End=\"0.2\" Mode=\"Set\" Start=\"0.1\" Track=\"%s\"\n" +
        "Silence:Use_Preset=\"<Factory Defaults>\"" +
        "SelectNone:\n" +
        "Select:End=\"0.2\" Mode=\"Set\" RelativeTo=\"ProjectStart\" Start=\"0\" Track=\"%s\"\n" +
        "Copy:\n" +
        "Paste:\n" +
        "Repeat:Count=\"49\"" + "\n" +
        "SelectTime:End=\"%s\" Start=\"%s\"" + "\n" +
        "Cut:\n"

String noiseTemplate = "NewMonoTrack:\n" +
        "Select:End=\"%s\" Mode=\"Set\" Start=\"0\" Track=\"%s\"\n" +
        "Noise:Amplitude=\"%f\" Type=\"White\"\n"

String exportTemplate = "SelectAll:\n" +
        "Export2:Filename=%s%s_%shz_%ssec_level%s.%s NumChannels=\"1\"\n" +
        "RemoveTracks:\n"


def showInputs = {
    show("Shock Yourself Macro Maker" +
            "\n\nGenerate audio files for 'Shock Yourself', 'Pain Stacks', and 'ShockJack'." +
            "\n\n" + helpTextMacro)
    showButton("Start")

    pathToAudacityMacros = getString("absolute path to Audacity macros or output path:", pathToAudacityMacros)
    if (pathToAudacityMacros[-1] != "/") pathToAudacityMacros += "/"
    macroName = getString("macro name:", macroName)
    outputPath = getString("sound files output path:", outputPath)
    if (outputPath[-1] != "/") outputPath += "/"

    show("waveforms:")
    waveforms.clear()
    while (waveforms.isEmpty()) {
        def waveformsCheckboxes = getBooleans("waveforms", WAVEFORMS, [true, true, true]);
        for (int i = 0; i < WAVEFORMS.size(); i++) {
            if (waveformsCheckboxes[i]) waveforms.add(WAVEFORMS[i])
        }
    }
    frequency = getInteger("frequency:", frequency)

    while (true) {
        noise = getFloat("noise (0.0 - 1.0):", noise)
        if (noise >= 0 && noise <= 1) break
    }

    makeBurst = getBoolean("Make sound files with burst?")
}

while (true) {
    try {
        showInputs()

        waveforms.each { String waveform ->

            // Waves
            fileName = waveform.toLowerCase()
            for (int level = minPainLevel; level <= maxPainLevel; level++) {
                amplitude = level * stepAmplitude + minAmplitude
                track = 0
                if (noise > 0) {
                    macroText += String.format(noiseTemplate, length, track, (amplitude * noise))
                    track++
                }
                macroText += String.format(waveTemplate, length, track, amplitude, frequency, waveform)
                macroText += String.format(exportTemplate, outputPath, fileName, frequency, length, level, extention)
            }

            // Burst
            if (makeBurst) {
                fileName = "burst_" + waveform.toLowerCase()
                for (int level = minPainLevel; level <= maxPainLevel; level++) {
                    amplitude = level * stepAmplitude + minAmplitude
                    track = 0
                    if (noise > 0) {
                        macroText += String.format(noiseTemplate, length, track, (amplitude * noise))
                        track++
                    }
                    macroText += String.format(burstTemplate, amplitude, frequency, waveform, track, track, (2 * length), length)
                    macroText += String.format(exportTemplate, outputPath, fileName, frequency, length, level, extention)
                }
            }
        }


        // macro file
        def macroFile = new File(pathToAudacityMacros + macroName)
        macroFile.write("")
        macroFile.append(macroText)


        def text = "Macro done:\n" + pathToAudacityMacros + macroName +
                "\n\n\n" + helpTextMacro +
                "\n\n\nSelect new eStim sound files in Shock Yourself under Change Variables!"
        show(text)
        showButton("->")
    } catch (e) {
        show("ERROR: " + e)
    }
}
